//
    Info<< "Reading the laser beam heat source properties\n" << endl;
   
    IOdictionary beamProperties
    (
        IOobject
        (
            "beamProperties",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );

//Weld (or laser beam) speed
    dimensionedVector Ulaser
    (
        "Ulaser",
        dimensionSet(0, 1, -1, 0, 0, 0, 0),
    	beamProperties.lookup("Ulaser")
    );

//Initial position of laser beam
    dimensionedVector Xbeam0
    (
        "Xbeam0",
        dimensionSet(0, 1, 0, 0, 0, 0, 0),
    	beamProperties.lookup("Xbeam0")
    );

//laser starting time
    dimensionedScalar timeStartLaser
    (
        "timeStartLaser",
        dimensionSet(0, 0, 1, 0, 0, 0, 0),
    	beamProperties.lookup("timeStartLaser") 
    );

//radius of laser beam 
    dimensionedScalar rBeam
    (
        "rBeam",
        dimensionSet(0, 1, 0, 0, 0, 0, 0),
    	beamProperties.lookup("rBeam") 
    );

//efficiency of laser beam 
    dimensionedScalar eta_L
    (
        "eta_L",
        dimensionSet(0, 0, 0, 0, 0, 0, 0),
    	beamProperties.lookup("eta_L")  
    );

//laser power 
    dimensionedScalar Q_L
    (
        "Q_L",
        dimensionSet(1, 2, -3, 0, 0, 0, 0),
    	beamProperties.lookup("Q_L")
    );



    // local position vector 
    const volVectorField& cellCentre = mesh.C();

    volScalarField r_local = Foam::sqrt(
                              Foam::sqr(cellCentre.component(vector::X)-(Xbeam0.component(vector::X)+(Ulaser.component(vector::X)*runTime.time())))
                             +Foam::sqr(cellCentre.component(vector::Z)-(Xbeam0.component(vector::Z)+(Ulaser.component(vector::Z)*runTime.time())))
                                       );


    //Info<< "Reading field Qhs \n" << endl;
    volScalarField Qhs
    (
        IOobject
        (
            "Qhs",
            runTime.timeName(),
            mesh,
            IOobject::READ_IF_PRESENT,
            IOobject::AUTO_WRITE
        ),
        mesh,
        dimensionedScalar
        (
             "Qhs",
             dimensionSet(1, 0, -3, 0, 0, 0, 0),
    	     scalar(0.0)
        )
    );

    // power density distribution factor
	scalar dl = 3.5;

	Qhs = eta_L*Q_L/(pi*Foam::sqr(rBeam))*dl*Foam::exp(-dl*(Foam::sqr(r_local)/Foam::sqr(rBeam)));
 
    // Info << " Qhs " << Qhs  << endl;

    Info<< "Create and initialise laser heat source Qhs done \n" << endl;
