Info<< "Reading field p_rgh\n" << endl;
volScalarField p_rgh
(
    IOobject
    (
        "p_rgh",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

Info<< "Reading field U\n" << endl;
volVectorField U
(
    IOobject
    (
        "U",
        runTime.timeName(),
        mesh,
        IOobject::MUST_READ,
        IOobject::AUTO_WRITE
    ),
    mesh
);

#include "createPhi.H"


Info<< "Reading transportProperties\n" << endl;
immiscibleIncompressibleTwoPhaseMixture mixture(U, phi);

volScalarField& alpha1(mixture.alpha1());
volScalarField& alpha2(mixture.alpha2());

const dimensionedScalar& rho1 = mixture.rho1();
const dimensionedScalar& rho2 = mixture.rho2();


// Need to store rho for ddt(rho, U)
volScalarField rho
(
    IOobject
    (
        "rho",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT
    ),
    alpha1*rho1 + alpha2*rho2,
    alpha1.boundaryField().types()
);
rho.oldTime();


//**************************************************************************************************//
    // for fields specific to the extension of interFoam to tInterFoam

    Info<< "Reading field T\n" << endl;
    volScalarField T
    (
        IOobject
        (
            "T",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );
    
    //--- read thermophysical propertiesa of phase 1

       Info<< "Reading thermophysicalProperties of phase 1\n" << endl;
    
       IOdictionary thermophysicalPropertiesPhase1
       (
           IOobject
           (
               "thermophysicalPropertiesPhase1",
               runTime.constant(),
               mesh,
               IOobject::MUST_READ,
               IOobject::NO_WRITE
           )
       );
    
 Info << "Creating pi \n" << endl;
 dimensionedScalar pi = constant::mathematical::pi;

//Only 1 Cp for phase1 since tehre is no phase change.

       const dimensionedScalar& cp1 = thermophysicalPropertiesPhase1.lookup("cp1");
       Info<< "Reading cp1l \n" << cp1 << endl;

//Only 1 kappa for phase1 since tehre is no phase change.

       // thermal conductivity
       const dimensionedScalar& kappa1 = thermophysicalPropertiesPhase1.lookup("kappa1");
       Info<< "Reading kappa1 \n" << kappa1 << endl;
    
    //--- end read therm0dynamic propertiesa of phase 1
    // ================================================
     
    
    // =============================================
    //--- read thermophysical propertiesa of phase 2

       Info<< "Reading thermophysicalProperties of phase 2\n" << endl;
    
       IOdictionary thermophysicalPropertiesPhase2
       (
           IOobject
           (
               "thermophysicalPropertiesPhase2",
               runTime.constant(),
               mesh,
               IOobject::MUST_READ,
               IOobject::NO_WRITE
           )
       );
    
       // density -> accessed via twoPhaseProperties (see below)
     
       // specific heat capacity
       const dimensionedScalar& cp2 = thermophysicalPropertiesPhase2.lookup("cp2");
       Info<< "Reading cp2 \n" << cp2 << endl;
       
       // thermal conductivity
       const dimensionedScalar& kappa2 = thermophysicalPropertiesPhase2.lookup("kappa2");
       Info<< "Reading kappa2 \n" << kappa2 << endl;
    
    //--- end read therm0dynamic propertiesa of phase 2
    // ================================================
     
      Info<< "Reading / initializing kappaf\n" <<endl;
      surfaceScalarField kappaf
      (
       	  IOobject
  	  (
	      "kappaf",
	      runTime.timeName(),
	      mesh,
	      IOobject::NO_READ,
	      IOobject::NO_WRITE
	  ),
          mesh,
	  dimensionedScalar("kappaf",dimensionSet(1,1,-3,-1,0,0,0), 0.0)
      );



    // updated and used in TEqn for ddt term;
    // so here we only set its dimension, the value is not used.
    Info<< "Reading / calculating rho*cp\n" <<endl;
    volScalarField rhoCp
    (
        IOobject
        (
            "rho*cp",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        alpha1*rho1*cp1+alpha2*rho2*cp2,
        alpha1.boundaryField().types()
    );
    rhoCp.oldTime();
    
    // updated and used in TEqn for convective term;
    // so here we only set its dimension, the value is not used.
    Info<< "Reading / calculating rhoPhi*cp\n" <<endl;
    surfaceScalarField rhoPhiCpf
    (
    	IOobject
	(
	      "rhoPhiCpf",
	      runTime.timeName(),
	      mesh,
	      IOobject::NO_READ,
	      IOobject::NO_WRITE
	),
        fvc::interpolate(rhoCp)*phi
    );

//*******************************************************************************//

// Mass flux
surfaceScalarField rhoPhi
(
    IOobject
    (
        "rhoPhi",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    fvc::interpolate(rho)*phi
);


// Construct incompressible turbulence model
autoPtr<incompressible::turbulenceModel> turbulence
(
    incompressible::turbulenceModel::New(U, phi, mixture)
);


// density weight for forces & fluxes applied at the liquid/vapor interface
volScalarField factor
(
    IOobject
    (
        "factor",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::NO_WRITE
    ),
    scalar(2)*rho/(rho1+rho2)
);


#include "readGravitationalAcceleration.H"
#include "readhRef.H"
#include "gh.H"


volScalarField p
(
    IOobject
    (
        "p",
        runTime.timeName(),
        mesh,
        IOobject::NO_READ,
        IOobject::AUTO_WRITE
    ),
    p_rgh + rho*gh
);

label pRefCell = 0;
scalar pRefValue = 0.0;
setRefCell
(
    p,
    p_rgh,
    pimple.dict(),
    pRefCell,
    pRefValue
);

if (p_rgh.needReference())
{
    p += dimensionedScalar
    (
        "p",
        p.dimensions(),
        pRefValue - getRefCellValue(p, pRefCell)
    );
    p_rgh = p - rho*gh;
}

mesh.setFluxRequired(p_rgh.name());
mesh.setFluxRequired(alpha1.name());

// MULES flux from previous time-step
surfaceScalarField alphaPhi
(
    IOobject
    (
        "alphaPhi",
        runTime.timeName(),
        mesh,
        IOobject::READ_IF_PRESENT,
        IOobject::AUTO_WRITE
    ),
    phi*fvc::interpolate(alpha1)
);

// MULES Correction
tmp<surfaceScalarField> talphaPhiCorr0;
