/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2013 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "localTranslatingFixedValueFvPatchField.H"
#include "mathematicalConstants.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class Type>
localTranslatingFixedValueFvPatchField<Type>::localTranslatingFixedValueFvPatchField
(
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF
)
:
    fixedValueFvPatchField<Type>(p, iF),
    dropInletValue_(p.size()),
    dropOutsideValue_(p.size()),
    dropCentre0_(p.size()),	
    dropCentre_(p.size()),
    dropRadius_(),
    faceCentres_(p.Cf()),
    //curTimeIndex_(-1),
    //dropFrequency_(),
    dropTranslatingVelocity_(),
    dropFrequency_()

{}

template<class Type>
localTranslatingFixedValueFvPatchField<Type>::localTranslatingFixedValueFvPatchField
(
    const localTranslatingFixedValueFvPatchField<Type>& ptf,
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchField<Type>(ptf, p, iF, mapper),
    dropInletValue_(ptf.dropInletValue_, mapper),
    dropOutsideValue_(ptf.dropOutsideValue_, mapper)
{}


template<class Type>
localTranslatingFixedValueFvPatchField<Type>::localTranslatingFixedValueFvPatchField
(
    const fvPatch& p,
    const DimensionedField<Type, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<Type>(p, iF),
    dropInletValue_("dropInletValue", dict, p.size()),
    dropOutsideValue_("dropOutsideValue", dict, p.size()),
    dropCentre0_(dict.lookup("dropCentre0")),
    dropCentre_(dict.lookup("dropCentre")),
    dropRadius_(dict.lookup("dropRadius")),
    faceCentres_(p.Cf()),
    dropTranslatingVelocity_(dict.lookup("dropTranslatingVelocity")),
    dropFrequency_(readScalar(dict.lookup("dropFrequency")))
{
    if (dict.found("value"))
    {
        fixedValueFvPatchField<Type>::operator==
        (
            Field<Type>("value", dict, p.size())
        );
    }
    else
    {
        fvPatchField<Type>::operator==(dropInletValue_);
	Field<Type>& patchfield = *this;
	forAll(p.Cf(),i)  		//go through all the cells in the current patch
    {
	bool isDrop=false;		
	forAll(dropRadius_,j)		//go through all the holes
	{
		if(mag(p.Cf()[i]-dropCentre_[j])<=dropRadius_[j]){isDrop=true;}		//includes all the cell inside teh radius of the droplet.
	
	{
		
		if(!isDrop){patchfield.data()[i]= dropOutsideValue_[i];}
	}
    }
}
}
}


template<class Type>
localTranslatingFixedValueFvPatchField<Type>::localTranslatingFixedValueFvPatchField
(
    const localTranslatingFixedValueFvPatchField<Type>& ptf
)
:
    fixedValueFvPatchField<Type>(ptf),
    dropInletValue_(ptf.dropInletValue_),
    dropOutsideValue_(ptf.dropOutsideValue_),
    dropCentre0_(ptf.dropCentre0_),	
    dropCentre_(ptf.dropCentre_),
    dropRadius_(ptf.dropRadius_),
    //faceCentres_(ptf.Cf())
    //curTimeIndex_(-1)
    dropTranslatingVelocity_(ptf.dropTranslatingVelocity_)
    //dropFrequency_(ptf.dropFrequency_),
{}


template<class Type>
localTranslatingFixedValueFvPatchField<Type>::localTranslatingFixedValueFvPatchField
(
    const localTranslatingFixedValueFvPatchField<Type>& ptf,
    const DimensionedField<Type, volMesh>& iF
)
:
    fixedValueFvPatchField<Type>(ptf, iF),
    dropInletValue_(ptf.dropInletValue_),
    dropOutsideValue_(ptf.dropOutsideValue_),
    dropCentre0_(ptf.dropCentre0_),	
    dropCentre_(ptf.dropCentre_),
    dropRadius_(ptf.dropRadius_),
    //faceCentres_(ptf.Cf())
    //curTimeIndex_(-1)
    dropTranslatingVelocity_(ptf.dropTranslatingVelocity_)
    //dropFrequency_(ptf.dropFrequency_),
{}

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //


template<class Type>
void localTranslatingFixedValueFvPatchField<Type>::autoMap
(
    const fvPatchFieldMapper& m
)
{
    fixedValueFvPatchField<Type>::autoMap(m);
    dropInletValue_.autoMap(m);
    dropOutsideValue_.autoMap(m);
    faceCentres_.autoMap(m);
}


template<class Type>
void localTranslatingFixedValueFvPatchField<Type>::rmap
(
    const fvPatchField<Type>& ptf,
    const labelList& addr
)
{
    fixedValueFvPatchField<Type>::rmap(ptf, addr);

    const localTranslatingFixedValueFvPatchField<Type>& tiptf =
        refCast<const localTranslatingFixedValueFvPatchField<Type> >(ptf);

    dropInletValue_.rmap(tiptf.dropInletValue_, addr);
    dropOutsideValue_.rmap(tiptf.dropOutsideValue_, addr);
    faceCentres_.rmap(tiptf.faceCentres_, addr);
}



template<class Type>
void localTranslatingFixedValueFvPatchField<Type>::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }
    
    scalar t_ = this->db().time().value();		//indirect access to the runTime.

    float tinj_ = 1/(2*dropFrequency_);

    const int ninj = floor(t_/tinj_);


    if (curTimeIndex_ != this->db().time().timeIndex())
    {
        Field<Type>& patchfield = *this;
	patchfield = dropInletValue_;
	
	forAll(this->faceCentres(),i)	//go through all the faces of the current patch
	{
		bool isDrop=false;
	 forAll(dropRadius_,j)		//go through all the inlets. Only 1 inlet in current case.
	{
	  if(mag(this->faceCentres()[i]-dropCentre_[j])<=dropRadius_[j]  && (t_ > 0.5 && ninj % 2 == 0)){isDrop=true;}	//Final report code
		}
	if(!isDrop){patchfield.data()[i]= dropOutsideValue_[i];}
	    }

    //const volVectorField& cellCentre = mesh.C();

        curTimeIndex_ = this->db().time().timeIndex();
	dropCentre_   = dropCentre0_ + dropTranslatingVelocity_ * t_;

        Info << "dropCentre at t0 is " << dropCentre0_ << endl;
        Info << "dropCentre at time t_ is " << dropCentre_ << endl;

    }

    fixedValueFvPatchField<Type>::updateCoeffs();
}




template<class Type>
void localTranslatingFixedValueFvPatchField<Type>::write(Ostream& os) const
{
    fixedValueFvPatchField<Type>::write(os);
    dropInletValue_.writeEntry("dropInletValue", os);
    dropOutsideValue_.writeEntry("dropOutsideValue", os);
    os.writeKeyword("dropCentre0")
	<< dropCentre0_ << token::END_STATEMENT << nl;
    os.writeKeyword("dropCentre")
	<< dropCentre_ << token::END_STATEMENT << nl;
    os.writeKeyword("dropRadius")
	<< dropRadius_ << token::END_STATEMENT << nl;
    os.writeKeyword("dropTranslatingVelocity")
	<< dropTranslatingVelocity_ << token::END_STATEMENT << nl;
    os.writeKeyword("dropFrequency")
	<< dropFrequency_ << token::END_STATEMENT << nl;
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
